@@ -45,10 +45,15 @@ from mercurial.node import hex
 
 from mercurial import (
     encoding,
+    pycompat,
     registrar,
     ui as uimod,
     util,
 )
+from mercurial.utils import (
+    dateutil,
+    procutil,
+)
 
 # Note for extension authors: ONLY specify testedwith = 'ships-with-hg-core' for
 # extensions which SHIP WITH MERCURIAL. Non-mainline extensions should
@@ -107,7 +112,7 @@ def _openlogfile(ui, vfs):
             if st.st_size >= maxsize:
                 path = vfs.join(name)
                 maxfiles = ui.configint('blackbox', 'maxfiles')
-                for i in xrange(maxfiles - 1, 1, -1):
+                for i in pycompat.xrange(maxfiles - 1, 1, -1):
                     rotate(oldpath='%s.%d' % (path, i - 1),
                            newpath='%s.%d' % (path, i))
                 rotate(oldpath=path,
@@ -164,9 +169,9 @@ def wrapui(ui):
                 return
             ui._bbinlog = True
             default = self.configdate('devel', 'default-date')
-            date = util.datestr(default, '%Y/%m/%d %H:%M:%S')
-            user = util.getuser()
-            pid = '%d' % util.getpid()
+            date = dateutil.datestr(default, '%Y/%m/%d %H:%M:%S')
+            user = procutil.getuser()
+            pid = '%d' % procutil.getpid()
             formattedmsg = msg[0] % msg[1:]
             rev = '(unknown)'
             changed = ''
@@ -221,10 +226,12 @@ def reposetup(ui, repo):
 
     repo._wlockfreeprefix.add('blackbox.log')
 
-@command('^blackbox',
+@command('blackbox',
     [('l', 'limit', 10, _('the number of events to show')),
     ],
-    _('hg blackbox [OPTION]...'))
+    _('hg blackbox [OPTION]...'),
+    helpcategory=command.CATEGORY_MAINTENANCE,
+    helpbasic=True)
 def blackbox(ui, repo, *revs, **opts):
     '''view the recent repository events
     '''
