@@ -91,7 +91,7 @@ tracked as largefiles::
   [largefiles]
   patterns =
     *.jpg
-    re:.*\.(png|bmp)$
+    re:.*\\.(png|bmp)$
     library.zip
     content/audio/*
 
@@ -109,6 +109,7 @@ from __future__ import absolute_import
 from mercurial import (
     hg,
     localrepo,
+    registrar,
 )
 
 from . import (
@@ -125,14 +126,27 @@ from . import (
 # leave the attribute unspecified.
 testedwith = 'ships-with-hg-core'
 
+configtable = {}
+configitem = registrar.configitem(configtable)
+
+configitem('largefiles', 'minsize',
+    default=configitem.dynamicdefault,
+)
+configitem('largefiles', 'patterns',
+    default=list,
+)
+configitem('largefiles', 'usercache',
+    default=None,
+)
+
 reposetup = reposetup.reposetup
 
 def featuresetup(ui, supported):
     # don't die on seeing a repo with the largefiles requirement
-    supported |= set(['largefiles'])
+    supported |= {'largefiles'}
 
 def uisetup(ui):
-    localrepo.localrepository.featuresetupfuncs.add(featuresetup)
+    localrepo.featuresetupfuncs.add(featuresetup)
     hg.wirepeersetupfuncs.append(proto.wirereposetup)
     uisetupmod.uisetup(ui)
 
