@@ -114,7 +114,7 @@ def getitemregister(configtable):
 coreconfigitem = getitemregister(coreitems)
 
 coreconfigitem('alias', '.*',
-    default=None,
+    default=dynamicdefault,
     generic=True,
 )
 coreconfigitem('annotate', 'nodates',
@@ -147,6 +147,9 @@ coreconfigitem('annotate', 'nobinary',
 coreconfigitem('annotate', 'noprefix',
     default=False,
 )
+coreconfigitem('annotate', 'word-diff',
+    default=False,
+)
 coreconfigitem('auth', 'cookiefile',
     default=None,
 )
@@ -158,10 +161,6 @@ coreconfigitem('bookmarks', 'pushing',
 coreconfigitem('bundle', 'mainreporoot',
     default='',
 )
-# bundle.reorder: experimental config
-coreconfigitem('bundle', 'reorder',
-    default='auto',
-)
 coreconfigitem('censor', 'policy',
     default='abort',
 )
@@ -184,6 +183,18 @@ coreconfigitem('color', 'mode',
 coreconfigitem('color', 'pagermode',
     default=dynamicdefault,
 )
+coreconfigitem('commands', 'grep.all-files',
+    default=False,
+)
+coreconfigitem('commands', 'resolve.confirm',
+    default=False,
+)
+coreconfigitem('commands', 'resolve.explicit-re-merge',
+    default=False,
+)
+coreconfigitem('commands', 'resolve.mark-check',
+    default='none',
+)
 coreconfigitem('commands', 'show.aliasprefix',
     default=list,
 )
@@ -193,13 +204,14 @@ coreconfigitem('commands', 'status.relative',
 coreconfigitem('commands', 'status.skipstates',
     default=[],
 )
+coreconfigitem('commands', 'status.terse',
+    default='',
+)
 coreconfigitem('commands', 'status.verbose',
     default=False,
 )
 coreconfigitem('commands', 'update.check',
     default=None,
-    # Deprecated, remove after 4.4 release
-    alias=[('experimental', 'updatecheck')]
 )
 coreconfigitem('commands', 'update.requiredest',
     default=False,
@@ -208,6 +220,9 @@ coreconfigitem('committemplate', '.*',
     default=None,
     generic=True,
 )
+coreconfigitem('convert', 'bzr.saverev',
+    default=True,
+)
 coreconfigitem('convert', 'cvsps.cache',
     default=True,
 )
@@ -362,6 +377,12 @@ coreconfigitem('devel', 'user.obsmarker',
 coreconfigitem('devel', 'warn-config-unknown',
     default=None,
 )
+coreconfigitem('devel', 'debug.copies',
+    default=False,
+)
+coreconfigitem('devel', 'debug.extensions',
+    default=False,
+)
 coreconfigitem('devel', 'debug.peer-request',
     default=False,
 )
@@ -395,6 +416,9 @@ coreconfigitem('diff', 'nobinary',
 coreconfigitem('diff', 'noprefix',
     default=False,
 )
+coreconfigitem('diff', 'word-diff',
+    default=False,
+)
 coreconfigitem('email', 'bcc',
     default=None,
 )
@@ -431,15 +455,24 @@ coreconfigitem('experimental', 'bundle2-output-capture',
 coreconfigitem('experimental', 'bundle2.pushback',
     default=False,
 )
-coreconfigitem('experimental', 'bundle2.stream',
-    default=False,
-)
 coreconfigitem('experimental', 'bundle2lazylocking',
     default=False,
 )
 coreconfigitem('experimental', 'bundlecomplevel',
     default=None,
 )
+coreconfigitem('experimental', 'bundlecomplevel.bzip2',
+    default=None,
+)
+coreconfigitem('experimental', 'bundlecomplevel.gzip',
+    default=None,
+)
+coreconfigitem('experimental', 'bundlecomplevel.none',
+    default=None,
+)
+coreconfigitem('experimental', 'bundlecomplevel.zstd',
+    default=None,
+)
 coreconfigitem('experimental', 'changegroup3',
     default=False,
 )
@@ -496,15 +529,18 @@ coreconfigitem('experimental', 'evolution.report-instabilities',
 coreconfigitem('experimental', 'evolution.track-operation',
     default=True,
 )
-coreconfigitem('experimental', 'worddiff',
-    default=False,
-)
 coreconfigitem('experimental', 'maxdeltachainspan',
     default=-1,
 )
+coreconfigitem('experimental', 'mergetempdirprefix',
+    default=None,
+)
 coreconfigitem('experimental', 'mmapindexthreshold',
     default=None,
 )
+coreconfigitem('experimental', 'narrow',
+    default=False,
+)
 coreconfigitem('experimental', 'nonnormalparanoidcheck',
     default=False,
 )
@@ -535,38 +571,62 @@ coreconfigitem('experimental', 'graphstyle.grandparent',
 coreconfigitem('experimental', 'hook-track-tags',
     default=False,
 )
-coreconfigitem('experimental', 'httppostargs',
+coreconfigitem('experimental', 'httppeer.advertise-v2',
     default=False,
 )
-coreconfigitem('experimental', 'manifestv2',
+coreconfigitem('experimental', 'httppeer.v2-encoder-order',
+    default=None,
+)
+coreconfigitem('experimental', 'httppostargs',
     default=False,
 )
 coreconfigitem('experimental', 'mergedriver',
     default=None,
 )
+coreconfigitem('experimental', 'nointerrupt', default=False)
+coreconfigitem('experimental', 'nointerrupt-interactiveonly', default=True)
+
 coreconfigitem('experimental', 'obsmarkers-exchange-debug',
     default=False,
 )
 coreconfigitem('experimental', 'remotenames',
     default=False,
 )
+coreconfigitem('experimental', 'removeemptydirs',
+    default=True,
+)
+coreconfigitem('experimental', 'revisions.prefixhexnode',
+    default=False,
+)
 coreconfigitem('experimental', 'revlogv2',
     default=None,
 )
+coreconfigitem('experimental', 'revisions.disambiguatewithin',
+    default=None,
+)
+coreconfigitem('experimental', 'server.filesdata.recommended-batch-size',
+    default=50000,
+)
+coreconfigitem('experimental', 'server.manifestdata.recommended-batch-size',
+    default=100000,
+)
+coreconfigitem('experimental', 'server.stream-narrow-clones',
+    default=False,
+)
 coreconfigitem('experimental', 'single-head-per-branch',
     default=False,
 )
-coreconfigitem('experimental', 'spacemovesdown',
+coreconfigitem('experimental', 'sshserver.support-v2',
     default=False,
 )
 coreconfigitem('experimental', 'sparse-read',
     default=False,
 )
 coreconfigitem('experimental', 'sparse-read.density-threshold',
-    default=0.25,
+    default=0.50,
 )
 coreconfigitem('experimental', 'sparse-read.min-gap-size',
-    default='256K',
+    default='65K',
 )
 coreconfigitem('experimental', 'treemanifest',
     default=False,
@@ -574,6 +634,24 @@ coreconfigitem('experimental', 'treemanifest',
 coreconfigitem('experimental', 'update.atomic-file',
     default=False,
 )
+coreconfigitem('experimental', 'sshpeer.advertise-v2',
+    default=False,
+)
+coreconfigitem('experimental', 'web.apiserver',
+    default=False,
+)
+coreconfigitem('experimental', 'web.api.http-v2',
+    default=False,
+)
+coreconfigitem('experimental', 'web.api.debugreflect',
+    default=False,
+)
+coreconfigitem('experimental', 'worker.wdir-get-thread-safe',
+    default=False,
+)
+coreconfigitem('experimental', 'xdiff',
+    default=False,
+)
 coreconfigitem('extensions', '.*',
     default=None,
     generic=True,
@@ -582,9 +660,6 @@ coreconfigitem('extdata', '.*',
     default=None,
     generic=True,
 )
-coreconfigitem('format', 'aggressivemergedeltas',
-    default=False,
-)
 coreconfigitem('format', 'chunkcachesize',
     default=None,
 )
@@ -598,11 +673,14 @@ coreconfigitem('format', 'manifestcachesize',
     default=None,
 )
 coreconfigitem('format', 'maxchainlen',
-    default=None,
+    default=dynamicdefault,
 )
 coreconfigitem('format', 'obsstore-version',
     default=None,
 )
+coreconfigitem('format', 'sparse-revlog',
+    default=False,
+)
 coreconfigitem('format', 'usefncache',
     default=True,
 )
@@ -612,6 +690,9 @@ coreconfigitem('format', 'usegeneraldelta',
 coreconfigitem('format', 'usestore',
     default=True,
 )
+coreconfigitem('format', 'internal-phase',
+    default=False,
+)
 coreconfigitem('fsmonitor', 'warn_when_unused',
     default=True,
 )
@@ -671,6 +752,11 @@ coreconfigitem('http_proxy', 'passwd',
 coreconfigitem('http_proxy', 'user',
     default=None,
 )
+
+coreconfigitem('http', 'timeout',
+    default=None,
+)
+
 coreconfigitem('logtoprocess', 'commandexception',
     default=None,
 )
@@ -704,6 +790,9 @@ coreconfigitem('merge', 'on-failure',
 coreconfigitem('merge', 'preferancestor',
         default=lambda: ['*'],
 )
+coreconfigitem('merge', 'strict-capability-check',
+    default=False,
+)
 coreconfigitem('merge-tools', '.*',
     default=None,
     generic=True,
@@ -743,6 +832,16 @@ coreconfigitem('merge-tools', br'.*\.gui$',
     generic=True,
     priority=-1,
 )
+coreconfigitem('merge-tools', br'.*\.mergemarkers$',
+    default='basic',
+    generic=True,
+    priority=-1,
+)
+coreconfigitem('merge-tools', br'.*\.mergemarkertemplate$',
+    default=dynamicdefault,  # take from ui.mergemarkertemplate
+    generic=True,
+    priority=-1,
+)
 coreconfigitem('merge-tools', br'.*\.priority$',
     default=0,
     generic=True,
@@ -823,6 +922,9 @@ coreconfigitem('profiling', 'sort',
 coreconfigitem('profiling', 'statformat',
     default='hotpath',
 )
+coreconfigitem('profiling', 'time-track',
+    default=dynamicdefault,
+)
 coreconfigitem('profiling', 'type',
     default='stat',
 )
@@ -859,6 +961,13 @@ coreconfigitem('progress', 'width',
 coreconfigitem('push', 'pushvars.server',
     default=False,
 )
+coreconfigitem('storage', 'new-repo-backend',
+    default='revlogv1',
+)
+coreconfigitem('storage', 'revlog.optimize-delta-parent-choice',
+    default=True,
+    alias=[('format', 'aggressivemergedeltas')],
+)
 coreconfigitem('server', 'bookmarks-pushkey-compat',
     default=True,
 )
@@ -880,6 +989,10 @@ coreconfigitem('server', 'bundle1.push',
 coreconfigitem('server', 'bundle1gd.push',
     default=None,
 )
+coreconfigitem('server', 'bundle2.stream',
+    default=True,
+    alias=[('experimental', 'bundle2.stream')]
+)
 coreconfigitem('server', 'compressionengines',
     default=list,
 )
@@ -892,9 +1005,15 @@ coreconfigitem('server', 'disablefullbundle',
 coreconfigitem('server', 'maxhttpheaderlen',
     default=1024,
 )
+coreconfigitem('server', 'pullbundle',
+    default=False,
+)
 coreconfigitem('server', 'preferuncompressed',
     default=False,
 )
+coreconfigitem('server', 'streamunbundle',
+    default=False,
+)
 coreconfigitem('server', 'uncompressed',
     default=True,
 )
@@ -907,6 +1026,9 @@ coreconfigitem('server', 'validate',
 coreconfigitem('server', 'zliblevel',
     default=-1,
 )
+coreconfigitem('server', 'zstdlevel',
+    default=3,
+)
 coreconfigitem('share', 'pool',
     default=None,
 )
@@ -1013,8 +1135,8 @@ coreconfigitem('ui', 'formatted',
 coreconfigitem('ui', 'graphnodetemplate',
     default=None,
 )
-coreconfigitem('ui', 'http2debuglevel',
-    default=None,
+coreconfigitem('ui', 'history-editing-backup',
+    default=True,
 )
 coreconfigitem('ui', 'interactive',
     default=None,
@@ -1025,6 +1147,9 @@ coreconfigitem('ui', 'interface',
 coreconfigitem('ui', 'interface.chunkselector',
     default=None,
 )
+coreconfigitem('ui', 'large-file-limit',
+    default=10000000,
+)
 coreconfigitem('ui', 'logblockedtimes',
     default=False,
 )
@@ -1078,6 +1203,9 @@ coreconfigitem('ui', 'report_untrusted',
 coreconfigitem('ui', 'rollback',
     default=True,
 )
+coreconfigitem('ui', 'signal-safe-lock',
+    default=True,
+)
 coreconfigitem('ui', 'slash',
     default=False,
 )
@@ -1114,9 +1242,6 @@ coreconfigitem('ui', 'traceback',
 coreconfigitem('ui', 'tweakdefaults',
     default=False,
 )
-coreconfigitem('ui', 'usehttp2',
-    default=False,
-)
 coreconfigitem('ui', 'username',
     alias=[('ui', 'user')]
 )
@@ -1149,6 +1274,9 @@ coreconfigitem('web', 'archivesubrepos',
 coreconfigitem('web', 'cache',
     default=True,
 )
+coreconfigitem('web', 'comparisoncontext',
+    default=5,
+)
 coreconfigitem('web', 'contact',
     default=None,
 )
@@ -1176,7 +1304,8 @@ coreconfigitem('web', 'accesslog',
 coreconfigitem('web', 'address',
     default='',
 )
-coreconfigitem('web', 'allow_archive',
+coreconfigitem('web', 'allow-archive',
+    alias=[('web', 'allow_archive')],
     default=list,
 )
 coreconfigitem('web', 'allow_read',
@@ -1242,6 +1371,12 @@ coreconfigitem('web', 'push_ssl',
 coreconfigitem('web', 'refreshinterval',
     default=20,
 )
+coreconfigitem('web', 'server-header',
+    default=None,
+)
+coreconfigitem('web', 'static',
+    default=None,
+)
 coreconfigitem('web', 'staticurl',
     default=None,
 )
