@@ -112,7 +112,6 @@ import json
 import os
 import signal
 import sys
-import tempfile
 import threading
 import time
 
@@ -126,13 +125,21 @@ contextmanager = contextlib.contextmanager
 
 __all__ = ['start', 'stop', 'reset', 'display', 'profile']
 
-skips = {"util.py:check", "extensions.py:closure",
-         "color.py:colorcmd", "dispatch.py:checkargs",
-         "dispatch.py:<lambda>", "dispatch.py:_runcatch",
-         "dispatch.py:_dispatch", "dispatch.py:_runcommand",
-         "pager.py:pagecmd", "dispatch.py:run",
-         "dispatch.py:dispatch", "dispatch.py:runcommand",
-         "hg.py:<module>", "evolve.py:warnobserrors",
+skips = {
+    r"util.py:check",
+    r"extensions.py:closure",
+    r"color.py:colorcmd",
+    r"dispatch.py:checkargs",
+    r"dispatch.py:<lambda>",
+    r"dispatch.py:_runcatch",
+    r"dispatch.py:_dispatch",
+    r"dispatch.py:_runcommand",
+    r"pager.py:pagecmd",
+    r"dispatch.py:run",
+    r"dispatch.py:dispatch",
+    r"dispatch.py:runcommand",
+    r"hg.py:<module>",
+    r"evolve.py:warnobserrors",
 }
 
 ###########################################################################
@@ -140,7 +147,7 @@ skips = {"util.py:check", "extensions.py:closure",
 
 def clock():
     times = os.times()
-    return times[0] + times[1]
+    return (times[0] + times[1], times[4])
 
 
 ###########################################################################
@@ -149,10 +156,11 @@ def clock():
 class ProfileState(object):
     def __init__(self, frequency=None):
         self.reset(frequency)
+        self.track = 'cpu'
 
     def reset(self, frequency=None):
         # total so far
-        self.accumulated_time = 0.0
+        self.accumulated_time = (0.0, 0.0)
         # start_time when timer is active
         self.last_start_time = None
         # a float
@@ -171,10 +179,23 @@ class ProfileState(object):
         self.samples = []
 
     def accumulate_time(self, stop_time):
-        self.accumulated_time += stop_time - self.last_start_time
+        increment = (
+            stop_time[0] - self.last_start_time[0],
+            stop_time[1] - self.last_start_time[1],
+        )
+        self.accumulated_time = (
+            self.accumulated_time[0] + increment[0],
+            self.accumulated_time[1] + increment[1],
+        )
 
     def seconds_per_sample(self):
-        return self.accumulated_time / len(self.samples)
+        return self.accumulated_time[self.timeidx] / len(self.samples)
+
+    @property
+    def timeidx(self):
+        if self.track == 'real':
+            return 1
+        return 0
 
 state = ProfileState()
 
@@ -236,6 +257,9 @@ class CodeSite(object):
     def filename(self):
         return os.path.basename(self.path)
 
+    def skipname(self):
+        return r'%s:%s' % (self.filename(), self.function)
+
 class Sample(object):
     __slots__ = (u'stack', u'time')
 
@@ -262,7 +286,8 @@ def profile_signal_handler(signum, frame):
         now = clock()
         state.accumulate_time(now)
 
-        state.samples.append(Sample.from_frame(frame, state.accumulated_time))
+        timestamp = state.accumulated_time[state.timeidx]
+        state.samples.append(Sample.from_frame(frame, timestamp))
 
         signal.setitimer(signal.ITIMER_PROF,
             state.sample_interval, 0.0)
@@ -275,7 +300,9 @@ def samplerthread(tid):
         state.accumulate_time(now)
 
         frame = sys._current_frames()[tid]
-        state.samples.append(Sample.from_frame(frame, state.accumulated_time))
+
+        timestamp = state.accumulated_time[state.timeidx]
+        state.samples.append(Sample.from_frame(frame, timestamp))
 
         state.last_start_time = now
         time.sleep(state.sample_interval)
@@ -289,8 +316,9 @@ def is_active():
     return state.profile_level > 0
 
 lastmechanism = None
-def start(mechanism='thread'):
+def start(mechanism='thread', track='cpu'):
     '''Install the profiling signal handler, and start profiling.'''
+    state.track = track # note: nesting different mode won't work
     state.profile_level += 1
     if state.profile_level == 1:
         state.last_start_time = clock()
@@ -333,18 +361,18 @@ def stop():
 
 def save_data(path):
     with open(path, 'w+') as file:
-        file.write(str(state.accumulated_time) + '\n')
+        file.write("%f %f\n" % state.accumulated_time)
         for sample in state.samples:
-            time = str(sample.time)
+            time = sample.time
             stack = sample.stack
-            sites = ['\1'.join([s.path, str(s.lineno), s.function])
+            sites = ['\1'.join([s.path, b'%d' % s.lineno, s.function])
                      for s in stack]
-            file.write(time + '\0' + '\0'.join(sites) + '\n')
+            file.write("%d\0%s\n" % (time, '\0'.join(sites)))
 
 def load_data(path):
     lines = open(path, 'r').read().splitlines()
 
-    state.accumulated_time = float(lines[0])
+    state.accumulated_time = [float(value) for value in lines[0].split()]
     state.samples = []
     for line in lines[1:]:
         parts = line.split('\0')
@@ -437,13 +465,14 @@ class DisplayFormats:
 
 def display(fp=None, format=3, data=None, **kwargs):
     '''Print statistics, either to stdout or the given file object.'''
-    data = data or state
+    if data is None:
+        data = state
 
     if fp is None:
         import sys
         fp = sys.stdout
     if len(data.samples) == 0:
-        print('No samples recorded.', file=fp)
+        fp.write(b'No samples recorded.\n')
         return
 
     if format == DisplayFormats.ByLine:
@@ -464,9 +493,9 @@ def display(fp=None, format=3, data=None, **kwargs):
         raise Exception("Invalid display format")
 
     if format not in (DisplayFormats.Json, DisplayFormats.Chrome):
-        print('---', file=fp)
-        print('Sample count: %d' % len(data.samples), file=fp)
-        print('Total time: %f seconds' % data.accumulated_time, file=fp)
+        fp.write(b'---\n')
+        fp.write(b'Sample count: %d\n' % len(data.samples))
+        fp.write(b'Total time: %f seconds (%f wall)\n' % data.accumulated_time)
 
 def display_by_line(data, fp):
     '''Print the profiler data with each sample line represented
@@ -474,34 +503,34 @@ def display_by_line(data, fp):
     stats = SiteStats.buildstats(data.samples)
     stats.sort(reverse=True, key=lambda x: x.selfseconds())
 
-    print('%5.5s %10.10s   %7.7s  %-8.8s' %
-          ('%  ', 'cumulative', 'self', ''), file=fp)
-    print('%5.5s  %9.9s  %8.8s  %-8.8s' %
-          ("time", "seconds", "seconds", "name"), file=fp)
+    fp.write(b'%5.5s %10.10s   %7.7s  %-8.8s\n' % (
+        b'%  ', b'cumulative', b'self', b''))
+    fp.write(b'%5.5s  %9.9s  %8.8s  %-8.8s\n' % (
+        b"time", b"seconds", b"seconds", b"name"))
 
     for stat in stats:
         site = stat.site
-        sitelabel = '%s:%d:%s' % (site.filename(), site.lineno, site.function)
-        print('%6.2f %9.2f %9.2f  %s' % (stat.selfpercent(),
-                                         stat.totalseconds(),
-                                         stat.selfseconds(),
-                                         sitelabel),
-              file=fp)
+        sitelabel = '%s:%d:%s' % (pycompat.fsencode(site.filename()),
+                                  site.lineno,
+                                  pycompat.sysbytes(site.function))
+        fp.write(b'%6.2f %9.2f %9.2f  %s\n' % (
+            stat.selfpercent(), stat.totalseconds(),
+            stat.selfseconds(), sitelabel))
 
 def display_by_method(data, fp):
     '''Print the profiler data with each sample function represented
     as one row in a table.  Important lines within that function are
     output as nested rows.  Sorted by self-time per line.'''
-    print('%5.5s %10.10s   %7.7s  %-8.8s' %
-          ('%  ', 'cumulative', 'self', ''), file=fp)
-    print('%5.5s  %9.9s  %8.8s  %-8.8s' %
-          ("time", "seconds", "seconds", "name"), file=fp)
+    fp.write(b'%5.5s %10.10s   %7.7s  %-8.8s\n' %
+          ('%  ', 'cumulative', 'self', ''))
+    fp.write(b'%5.5s  %9.9s  %8.8s  %-8.8s\n' %
+          ("time", "seconds", "seconds", "name"))
 
     stats = SiteStats.buildstats(data.samples)
 
     grouped = defaultdict(list)
     for stat in stats:
-        grouped[stat.site.filename() + ":" + stat.site.function].append(stat)
+        grouped[stat.site.filename() + r":" + stat.site.function].append(stat)
 
     # compute sums for each function
     functiondata = []
@@ -526,20 +555,24 @@ def display_by_method(data, fp):
     for function in functiondata:
         if function[3] < 0.05:
             continue
-        print('%6.2f %9.2f %9.2f  %s' % (function[3], # total percent
-                                         function[1], # total cum sec
-                                         function[2], # total self sec
-                                         function[0]), # file:function
-              file=fp)
+        fp.write(b'%6.2f %9.2f %9.2f  %s\n' % (
+            function[3], # total percent
+            function[1], # total cum sec
+            function[2], # total self sec
+            pycompat.sysbytes(function[0]))) # file:function
+
         function[4].sort(reverse=True, key=lambda i: i.selfseconds())
         for stat in function[4]:
             # only show line numbers for significant locations (>1% time spent)
             if stat.selfpercent() > 1:
                 source = stat.site.getsource(25)
+                if sys.version_info.major >= 3 and not isinstance(source, bytes):
+                    source = pycompat.bytestr(source)
+
                 stattuple = (stat.selfpercent(), stat.selfseconds(),
                              stat.site.lineno, source)
 
-                print('%33.0f%% %6.2f   line %s: %s' % (stattuple), file=fp)
+                fp.write(b'%33.0f%% %6.2f   line %d: %s\n' % stattuple)
 
 def display_about_method(data, fp, function=None, **kwargs):
     if function is None:
@@ -573,9 +606,12 @@ def display_about_method(data, fp, function=None, **kwargs):
     parents = [(parent, count) for parent, count in parents.iteritems()]
     parents.sort(reverse=True, key=lambda x: x[1])
     for parent, count in parents:
-        print('%6.2f%%   %s:%s   line %s: %s' %
-            (count / relevant_samples * 100, parent.filename(),
-            parent.function, parent.lineno, parent.getsource(50)), file=fp)
+        fp.write(b'%6.2f%%   %s:%s   line %s: %s\n' %
+            (count / relevant_samples * 100,
+             pycompat.fsencode(parent.filename()),
+             pycompat.sysbytes(parent.function),
+             parent.lineno,
+             pycompat.sysbytes(parent.getsource(50))))
 
     stats = SiteStats.buildstats(data.samples)
     stats = [s for s in stats
@@ -592,23 +628,23 @@ def display_about_method(data, fp, function=None, **kwargs):
         total_self_percent += stat.selfpercent()
         total_cum_percent += stat.totalpercent()
 
-    print(
-        '\n    %s:%s    Total: %0.2fs (%0.2f%%)    Self: %0.2fs (%0.2f%%)\n' %
-        (
-        filename or '___',
-        function,
+    fp.write(
+        b'\n    %s:%s    Total: %0.2fs (%0.2f%%)    Self: %0.2fs (%0.2f%%)\n\n'
+        % (
+        pycompat.sysbytes(filename or '___'),
+        pycompat.sysbytes(function),
         total_cum_sec,
         total_cum_percent,
         total_self_sec,
         total_self_percent
-        ), file=fp)
+        ))
 
     children = [(child, count) for child, count in children.iteritems()]
     children.sort(reverse=True, key=lambda x: x[1])
     for child, count in children:
-        print('        %6.2f%%   line %s: %s' %
+        fp.write(b'        %6.2f%%   line %s: %s\n' %
               (count / relevant_samples * 100, child.lineno,
-               child.getsource(50)), file=fp)
+               pycompat.sysbytes(child.getsource(50))))
 
 def display_hotpath(data, fp, limit=0.05, **kwargs):
     class HotNode(object):
@@ -628,7 +664,7 @@ def display_hotpath(data, fp, limit=0.05, **kwargs):
             if len(stack) > 1:
                 i = 1
                 # Skip boiler plate parts of the stack
-                while i < len(stack) and '%s:%s' % (stack[i].filename(), stack[i].function) in skips:
+                while i < len(stack) and stack[i].skipname() in skips:
                     i += 1
                 if i < len(stack):
                     child.add(stack[i:], time)
@@ -669,7 +705,7 @@ def display_hotpath(data, fp, limit=0.05, **kwargs):
             # Make frames that didn't actually perform work dark grey
             elif node.count - childrensamples == 0:
                 finalstring = '\033[90m' + finalstring + '\033[0m'
-            print(finalstring, file=fp)
+            fp.write(finalstring + b'\n')
 
         newdepth = depth
         if len(visiblechildren) > 1 or multiple_siblings:
@@ -686,12 +722,11 @@ def write_to_flame(data, fp, scriptpath=None, outputfile=None, **kwargs):
     if scriptpath is None:
         scriptpath = encoding.environ['HOME'] + '/flamegraph.pl'
     if not os.path.exists(scriptpath):
-        print("error: missing %s" % scriptpath, file=fp)
-        print("get it here: https://github.com/brendangregg/FlameGraph",
-              file=fp)
+        fp.write(b'error: missing %s\n' % scriptpath)
+        fp.write(b'get it here: https://github.com/brendangregg/FlameGraph\n')
         return
 
-    fd, path = tempfile.mkstemp()
+    fd, path = pycompat.mkstemp()
 
     file = open(path, "w+")
 
@@ -706,7 +741,7 @@ def write_to_flame(data, fp, scriptpath=None, outputfile=None, **kwargs):
             lines[line] = 1
 
     for line, count in lines.iteritems():
-        file.write("%s %s\n" % (line, count))
+        file.write("%s %d\n" % (line, count))
 
     file.close()
 
@@ -714,7 +749,7 @@ def write_to_flame(data, fp, scriptpath=None, outputfile=None, **kwargs):
         outputfile = '~/flamegraph.svg'
 
     os.system("perl ~/flamegraph.pl %s > %s" % (path, outputfile))
-    print("Written to %s" % outputfile, file=fp)
+    fp.write(b'Written to %s\n' % outputfile)
 
 _pathcache = {}
 def simplifypath(path):
@@ -744,7 +779,11 @@ def write_to_json(data, fp):
 
         samples.append((sample.time, stack))
 
-    print(json.dumps(samples), file=fp)
+    data = json.dumps(samples)
+    if not isinstance(data, bytes):
+        data = data.encode('utf-8')
+
+    fp.write(data)
 
 def write_to_chrome(data, fp, minthreshold=0.005, maxthreshold=0.999):
     samples = []
@@ -842,7 +881,7 @@ def write_to_chrome(data, fp, minthreshold=0.005, maxthreshold=0.999):
     fp.write('\n')
 
 def printusage():
-    print("""
+    print(r"""
 The statprof command line allows you to inspect the last profile's results in
 the following forms:
 
@@ -873,17 +912,17 @@ def main(argv=None):
 
     optstart = 2
     displayargs['function'] = None
-    if argv[1] == 'hotpath':
+    if argv[1] == r'hotpath':
         displayargs['format'] = DisplayFormats.Hotpath
-    elif argv[1] == 'lines':
+    elif argv[1] == r'lines':
         displayargs['format'] = DisplayFormats.ByLine
-    elif argv[1] == 'functions':
+    elif argv[1] == r'functions':
         displayargs['format'] = DisplayFormats.ByMethod
-    elif argv[1] == 'function':
+    elif argv[1] == r'function':
         displayargs['format'] = DisplayFormats.AboutMethod
         displayargs['function'] = argv[2]
         optstart = 3
-    elif argv[1] == 'flame':
+    elif argv[1] == r'flame':
         displayargs['format'] = DisplayFormats.FlameGraph
     else:
         printusage()
@@ -901,22 +940,22 @@ def main(argv=None):
     displayargs['limit'] = 0.05
     path = None
     for o, value in opts:
-        if o in ("-l", "--limit"):
+        if o in (r"-l", r"--limit"):
             displayargs['limit'] = float(value)
-        elif o in ("-f", "--file"):
+        elif o in (r"-f", r"--file"):
             path = value
-        elif o in ("-o", "--output-file"):
+        elif o in (r"-o", r"--output-file"):
             displayargs['outputfile'] = value
-        elif o in ("-p", "--script-path"):
+        elif o in (r"-p", r"--script-path"):
             displayargs['scriptpath'] = value
-        elif o in ("-h", "help"):
+        elif o in (r"-h", r"help"):
             printusage()
             return 0
         else:
             assert False, "unhandled option %s" % o
 
     if not path:
-        print('must specify --file to load')
+        print(r'must specify --file to load')
         return 1
 
     load_data(path=path)
@@ -925,5 +964,5 @@ def main(argv=None):
 
     return 0
 
-if __name__ == "__main__":
+if __name__ == r"__main__":
     sys.exit(main())
