@@ -5,22 +5,22 @@
 from __future__ import absolute_import
 
 from mercurial import (
-    cmdutil,
     merge,
+    registrar,
 )
 
 cmdtable = {}
-command = cmdutil.command(cmdtable)
+command = registrar.command(cmdtable)
 
-@command('fakemergerecord',
-         [('X', 'mandatory', None, 'add a fake mandatory record'),
-          ('x', 'advisory', None, 'add a fake advisory record')], '')
+@command(b'fakemergerecord',
+         [(b'X', b'mandatory', None, b'add a fake mandatory record'),
+          (b'x', b'advisory', None, b'add a fake advisory record')], '')
 def fakemergerecord(ui, repo, *pats, **opts):
     with repo.wlock():
         ms = merge.mergestate.read(repo)
         records = ms._makerecords()
         if opts.get('mandatory'):
-            records.append(('X', 'mandatory record'))
+            records.append((b'X', b'mandatory record'))
         if opts.get('advisory'):
-            records.append(('x', 'advisory record'))
+            records.append((b'x', b'advisory record'))
         ms._writerecords(records)
