@@ -1,15 +1,15 @@
 from __future__ import absolute_import
-from mercurial import (
-    util,
+from mercurial.utils import (
+    procutil,
 )
 
 # XXX: we should probably offer a devel option to do this in blackbox directly
 def getuser():
-    return 'bob'
+    return b'bob'
 def getpid():
     return 5000
 
 # mock the date and user apis so the output is always the same
 def uisetup(ui):
-    util.getuser = getuser
-    util.getpid = getpid
+    procutil.getuser = getuser
+    procutil.getpid = getpid
