@@ -7,8 +7,12 @@
 #   seq START STOP        [START, STOP] stepping by 1
 #   seq START STEP STOP   [START, STOP] stepping by STEP
 
+from __future__ import absolute_import, print_function
 import sys
 
+if sys.version_info[0] >= 3:
+    xrange = range
+
 start = 1
 if len(sys.argv) > 2:
     start = int(sys.argv[1])
@@ -20,4 +24,4 @@ if len(sys.argv) > 3:
 stop = int(sys.argv[-1]) + 1
 
 for i in xrange(start, stop, step):
-    print i
+    print(i)
